// ktDibJpeg.cpp: CktDibJpeg NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////
/*/////////////////////////////////////////////////////////////////////
//DD Jpeg
//Copyright (C) 2001 Kobataku
//mailto:null@basil.freemail.ne.jp
//HP:http://hp.vector.co.jp/authors/VA021021/						
/////////////////////////////////////////////////////////////////////*/

#include "stdafx.h"
#include "dd_jpeg.h"
#include "ktDibJpeg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CktDibJpeg::CktDibJpeg() {}
CktDibJpeg::~CktDibJpeg() {DeleteDib();}

bool CktDibJpeg::LoadJpegFile(LPCTSTR pszPath) 
{
    JPEG_CORE_PROPERTIES image;
    ZeroMemory( &image, sizeof( JPEG_CORE_PROPERTIES ) );
    BYTE* imageData;
	int nBitDepth=24;
    try {
        if( ijlInit( &image ) != IJL_OK )
        {
            TRACE( "Cannot initialize Intel JPEG library\n" );
            throw;
        }
		
        image.JPGFile = const_cast<char*>(pszPath);
        if( ijlRead( &image, IJL_JFILE_READPARAMS ) != IJL_OK )
        {
            TRACE( "Cannot read JPEG file header from %s file\n",
              image.JPGFile );
            throw;
        }
        // Set the JPG color space ... this will always be
        // somewhat of an educated guess at best because JPEG
        // is "color blind" (i.e., nothing in the bit stream
        // tells you what color space the data was encoded from).
        // However, in this example we assume that we are
        // reading JFIF files which means that 3 channel images
        // are in the YCbCr color space and 1 channel images are
        // in the Y color space.
        switch(image.JPGChannels)
		{
		case 1:
			image.JPGColor    = IJL_G;
			image.DIBChannels = 3;
			image.DIBColor    = IJL_BGR;
			nBitDepth=24;
			break;

		case 3:
			image.JPGColor    = IJL_YCBCR;
			image.DIBChannels = 3;
			image.DIBColor    = IJL_BGR;
			nBitDepth=24;
			break;
		case 4:
			image.JPGColor    = IJL_YCBCRA_FPX;
			image.DIBChannels = 4;
			image.DIBColor    = IJL_RGBA_FPX;
			nBitDepth=32;
			break;
		default:
		// This catches everything else, but no
		// color twist will be performed by the IJL.
			image.DIBColor = (IJL_COLOR)IJL_OTHER;
			image.JPGColor = (IJL_COLOR)IJL_OTHER;
			image.DIBChannels = image.JPGChannels;
		}
        image.DIBWidth    = image.JPGWidth;
        image.DIBHeight   = image.JPGHeight;
        image.DIBPadBytes = IJL_DIB_PAD_BYTES(image.DIBWidth,image.DIBChannels);

        int imageSize = (image.DIBWidth * image.DIBChannels + image.DIBPadBytes) *
          image.DIBHeight;

        imageData = new BYTE[ imageSize ];
        if( imageData == NULL )
        {
            TRACE( "Cannot allocate memory for image\n" );
			throw CString("cannot allocate");
        }
        image.DIBBytes = imageData;

        if( ijlRead( &image, IJL_JFILE_READWHOLEIMAGE ) != IJL_OK )
        {
            TRACE( "Cannot read image data from %s file\n", image.JPGFile );
            delete[] imageData;
            throw CString("cannot read image");
        }

        if( ijlFree( &image ) != IJL_OK )
        {
            TRACE( "Cannot free Intel(R) JPEG library" );
        }

        if(image.DIBColor == IJL_RGBA_FPX)
        {
			RGBA_FPX_to_BGRA(imageData,image.DIBWidth,image.DIBHeight);
        }
		if( CreateDib(image.DIBWidth, -image.DIBHeight, nBitDepth, imageData) == false )
			throw CString("Cannot Create dib");
	}
    catch(...) {
		ijlFree( &image );
        //AfxMessageBox( "JPEG" );
        return false;
    }
	return true;
}

void CktDibJpeg::RGBA_FPX_to_BGRA(BYTE* data,int width,int height)
{
	int   i,j;
	int   pad;
	int   line_width;
	BYTE  r, g, b, a;
	BYTE* ptr;

	ptr = data;
	pad = IJL_DIB_PAD_BYTES(width,4);
	line_width = width * 4 + pad;

	for(i = 0; i < height; i++)
	{
		ptr = data + line_width*i;
		for(j = 0; j < width; j++)
		{
			r = ptr[0];
			g = ptr[1];
			b = ptr[2];
			a = ptr[3];
			ptr[2] = (BYTE)( (r*a+1) >> 8 );
			ptr[1] = (BYTE)( (g*a+1) >> 8 );
			ptr[0] = (BYTE)( (b*a+1) >> 8 );
			ptr += 4;
		}
	}

	return;
} 

bool CktDibJpeg::SaveJpegFile(LPCTSTR lpszPathName, int nQuality/*=75*/)
{
    bool bres = true;
    JPEG_CORE_PROPERTIES image;
    ZeroMemory( &image, sizeof( JPEG_CORE_PROPERTIES ) );
	BITMAP bm; ZeroMemory(&bm ,sizeof(BITMAP));

	try {
		GetBitmap(&bm);
		if( ijlInit( &image ) != IJL_OK )
        {
            TRACE( "Can't initialize Intel(R) JPEG library\n" );
			throw CString("Can't initilize library");
        }

        // Setup DIB
        image.DIBWidth         = bm.bmWidth;
        image.DIBHeight        = bm.bmHeight;
        image.DIBBytes         = (BYTE*)bm.bmBits;

        // Setup JPEG
        image.JPGFile          = const_cast<char*>(lpszPathName);
        image.JPGWidth         = bm.bmWidth;
        image.JPGHeight        = bm.bmHeight;
	
        switch(bm.bmBitsPixel) {
        case 3:
			image.DIBColor       = IJL_BGR;
			image.DIBChannels    = 3;
			image.DIBPadBytes    = IJL_DIB_PAD_BYTES(image.DIBWidth,3);
			image.JPGColor       = IJL_YCBCR;
			image.JPGChannels    = 3;
			image.JPGSubsampling = IJL_411;
			break;

        case 4:
			image.DIBColor       = IJL_RGBA_FPX;
			image.DIBChannels    = 4;
			image.DIBPadBytes    = IJL_DIB_PAD_BYTES(image.DIBWidth,4);
			image.JPGColor       = IJL_YCBCRA_FPX;
			image.JPGChannels    = 4;
			image.JPGSubsampling = IJL_4114;
          break;

        default:
			break;
        }

        if(image.DIBColor == IJL_RGBA_FPX)
        {
			BGRA_to_RGBA(image.DIBBytes, image.DIBWidth, image.DIBHeight);
        }

		image.jquality = nQuality;

        if( ijlWrite( &image, IJL_JFILE_WRITEWHOLEIMAGE ) != IJL_OK )
        {
            TRACE( "Can't write image\n" );
            throw CString("Can't write image");
        }

        if( ijlFree( &image ) != IJL_OK )
        {
            TRACE( "Can't free Intel(R) JPEG library\n" );
			throw CString("Can't free library");
        }
	}
	catch(...) {
		bres = false;
		ijlFree( &image );
	}
    return bres;
}

void CktDibJpeg::BGRA_to_RGBA(BYTE* data,int width,int height)
{
  int   i;
  int   j;
  int   pad;
  int   line_width;
  BYTE  r, g, b, a;
  BYTE* ptr;

  ptr = data;
  pad = IJL_DIB_PAD_BYTES(width,4);
  line_width = width * 4 + pad;

  for(i = 0; i < height; i++)
  {
    ptr = data + line_width*i;
    for(j = 0; j < width; j++)
    {
      b = ptr[0];
      g = ptr[1];
      r = ptr[2];
      a = ptr[3];
      ptr[0] = r;
      ptr[1] = g;
      ptr[2] = b;
      ptr += 4;
    }
  }

  return;
}
//㉺ЂԂ
void CktDibJpeg::ChangeTopDown()
{
	BITMAP bm; ZeroMemory(&bm, sizeof(BITMAP));
	GetBitmap(&bm);
	int wb = bm.bmWidthBytes;
	char* c = new char[wb*bm.bmHeight];
	char* wk = new char[wb];
	int nTop=0,nDown=bm.bmHeight-1;
	BYTE*bp = (BYTE*)bm.bmBits;
	while(1) {
		memcpy(wk, &bp[nTop*wb], wb);
		memcpy(&bp[nTop*wb], &bp[nDown*wb], wb);
		memcpy(&bp[nDown*wb], wk, wb);

		if( nDown-nTop <= 1)
			break;
		--nDown; ++nTop;
	}
	delete c;
	delete wk;
}
